
# This file was automatically generated by Dist::Zilla::Plugin::ModuleBuild v6.012.
use strict;
use warnings;

use Module::Build 0.28;


my %module_build_args = (
  "build_requires" => {
    "Module::Build" => "0.28"
  },
  "configure_requires" => {
    "Module::Build" => "0.28"
  },
  "dist_abstract" => "a cpanplus backend to build Fedora/RedHat rpms",
  "dist_author" => [
    "Shlomi Fish <shlomif\@cpan.org>"
  ],
  "dist_name" => "CPANPLUS-Dist-Fedora",
  "dist_version" => "0.2.2",
  "license" => "perl",
  "module_name" => "CPANPLUS::Dist::Fedora",
  "recursive_test_files" => 1,
  "requires" => {
    "CPANPLUS::Dist::Base" => 0,
    "CPANPLUS::Error" => 0,
    "Cwd" => 0,
    "File::Basename" => 0,
    "File::Copy" => 0,
    "IPC::Cmd" => 0,
    "List::Util" => 0,
    "POSIX" => 0,
    "Pod::POM" => 0,
    "Pod::POM::View::Text" => 0,
    "Template" => 0,
    "Text::Wrap" => 0,
    "parent" => 0,
    "strict" => 0,
    "warnings" => 0
  },
  "test_requires" => {
    "File::Spec" => 0,
    "IO::Handle" => 0,
    "IPC::Open3" => 0,
    "Test::More" => "0.88",
    "perl" => "5.006"
  }
);


my %fallback_build_requires = (
  "File::Spec" => 0,
  "IO::Handle" => 0,
  "IPC::Open3" => 0,
  "Module::Build" => "0.28",
  "Test::More" => "0.88",
  "perl" => "5.006"
);


unless ( eval { Module::Build->VERSION(0.4004) } ) {
  delete $module_build_args{test_requires};
  $module_build_args{build_requires} = \%fallback_build_requires;
}

my $build = Module::Build->new(%module_build_args);


$build->create_build_script;
